#!/bin/bash
# Copyright (c) Daniel Sheffield 2024 - 2025

set -euo pipefail
if [ "$#" == "0" ]
then
	echo capture
	exit 0
fi

. "${BASH_SOURCE[0]%/*}"/helpers/common.sh

PROG="$1"
SUBPROG="$(get_subprog "$@")"

TIMESTAMP=$(date +%Y%m%d_%H%M%S_%N)
LOG_DIR="$LOG_ROOT"/"$PROG"/"$SUBPROG"/"$TIMESTAMP"

mkdir -p "$LOG_DIR"

PTY="$LOG_DIR"/pty
OUT="$LOG_DIR"/stdout
ERR="$LOG_DIR"/stderr
DAT="$LOG_DIR"/dat
INF="$LOG_DIR"/info
CMD="$(shell_escape "$@")"

info(){
	cat <<EOF > "$INF"
$CMD
status=$?
EOF
	exec 1>&- 2>&-
	[ "${socat:-}" ] && kill "$socat"
	wait
}

trap 'info' EXIT

exec 2> >(tee -a "$DAT" "$ERR")
if which socat 1>/dev/null 2>&1
then
	tty="$(tty)"
	socat PTY,rawer,link="$PTY" EXEC:"$(shell_escape tee -a "$DAT" "$OUT" "$tty")" &
	socat="$!"
	while ! [ -e "$PTY" ]; do sleep 0.01; done
	stty -F "$PTY" rows "$LINES" cols "$COLUMNS"
	exec 1> "$PTY"
else
	exec 1> >(tee -a "$DAT" "$OUT")
fi
"$@"
